import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';


import 'package:stripe_package/flutterwave222/core/metrics/metric_manager.dart';
import 'package:stripe_package/flutterwave222/models/requests/pay_with_bank_account/pay_with_bank_account.dart';
import 'package:stripe_package/flutterwave222/models/responses/charge_response.dart';
import 'package:stripe_package/flutterwave222/utils/flutterwave_urls.dart';
import 'package:http/http.dart' as http;




class BankAccountPaymentManager {
  String publicKey;
  String currency;
  String amount;
  String email;
  String txRef;
  String flwref;
  bool isDebugMode;
  String phoneNumber;
  String? accountBank;
  String? accountNumber;
  String fullName;
  String customer;
  String? redirectUrl;

  /// BankAccountPaymentManager constructor
  /// Available for only payments with NGN currency
  BankAccountPaymentManager({
    required this.publicKey,
    required this.currency,
    required this.amount,
    required this.email,
    required this.customer,
    required this.txRef,
    required this.flwref,
    required this.isDebugMode,
    required this.phoneNumber,
    required this.fullName,
    this.accountBank,
    this.accountNumber,
    this.redirectUrl,
  });

  /// Initiates payments via Bank Account
  /// Available for only payments with NGN currency
  /// returns an instance of ChargeResponse or throws an error
  Future<ChargeResponse> payWithAccount(
      BankAccountPaymentRequest bankAccountRequest, http.Client client) async {
    final stopWatch = Stopwatch();
    final requestBody = bankAccountRequest.toJson();

    // final url = FlutterwaveURLS.getBaseUrl(this.isDebugMode) +
    //     FlutterwaveURLS.PAY_WITH_ACCOUNT;
    final url = "https://api.flutterwave.com/v3/transfers";

    final uri = Uri.parse(url);
    try {
      final http.Response response = await client.post(uri,
          headers: {
            // HttpHeaders.authorizationHeader: this.publicKey,
            HttpHeaders.authorizationHeader: "FLWSECK_TEST-e358b24a0479c478d66ccbe957b3b935-X",
            HttpHeaders.contentTypeHeader: "application/json"
          },
          body: jsonEncode(requestBody));

      MetricManager.logMetric(
          client,
          "FLWSECK_TEST-e358b24a0479c478d66ccbe957b3b935-X",
          // seck key,
          MetricManager.INITIATE_ACCOUNT_CHARGE,
          "${stopWatch.elapsedMilliseconds}ms");

      ChargeResponse chargeResponse =
          ChargeResponse.fromJson(json.decode(response.body));


      return chargeResponse;


    } catch (error) {

      MetricManager.logMetric(
          client,
          "FLWSECK_TEST-e358b24a0479c478d66ccbe957b3b935-X",
          // publicKey,
          MetricManager.INITIATE_ACCOUNT_CHARGE_ERROR,
          "${stopWatch.elapsedMilliseconds}ms");
      throw (FlutterError(error.toString()));
    }

  }
}
