import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'dart:convert';
import 'package:http/http.dart' as http;




class HistoryAppp extends StatelessWidget {
  const HistoryAppp({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        // Remove the debug banner
        debugShowCheckedModeBanner: false,
        theme: ThemeData(
          primarySwatch: Colors.lightBlue,
        ),
        home: const HomePage());
  }
}

class HomePage extends StatefulWidget {
  const HomePage({Key? key}) : super(key: key);

  @override
  State<HomePage> createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  final _baseUrl = 'https://api.flutterwave.com/v3/transfers';

  int _page = 1;


  bool _isFirstLoadRunning = false;
  bool _hasNextPage = true;

  bool _isLoadMoreRunning = false;

  List _posts = [];

  void _loadMore() async {
    if (_hasNextPage == true &&
        _isFirstLoadRunning == false &&
        _isLoadMoreRunning == false &&
        _controller.position.extentAfter < 300) {
      setState(() {
        _isLoadMoreRunning = true; // Display a progress indicator at the bottom
      });

      _page += 1; // Increase _page by 1

      try {
        final res = await http.get(
          Uri.parse("$_baseUrl?page=$_page"),
          headers: {
            HttpHeaders.authorizationHeader:
                "FLWSECK_TEST-e358b24a0479c478d66ccbe957b3b935-X",
            HttpHeaders.contentTypeHeader: 'application/json'
          },
        );

        final List fetchedPosts = json.decode(res.body)["data"];
        if (fetchedPosts.isNotEmpty) {
          setState(() {
            _posts.addAll(fetchedPosts);
          });
        } else {
          setState(() {
            _hasNextPage = false;
          });
        }
      } catch (err) {
        if (kDebugMode) {
          print('Something went wrong!');
        }
      }

      setState(() {
        _isLoadMoreRunning = false;
      });
    }
  }

  void _firstLoad() async {
    setState(() {
      _isFirstLoadRunning = true;
    });

    try {
      final res = await http.get(
        Uri.parse("$_baseUrl?_page=$_page"),
        headers: {
          HttpHeaders.authorizationHeader:
              "FLWSECK_TEST-e358b24a0479c478d66ccbe957b3b935-X",
          HttpHeaders.contentTypeHeader: 'application/json'
        },
      );
      setState(() {
        _posts = json.decode(res.body)["data"];
      });
    } catch (err) {
      if (kDebugMode) {
        print('Something went wrong');
      }
    }

    setState(() {
      _isFirstLoadRunning = false;
    });
  }

  late ScrollController _controller;
  @override
  void initState() {
    super.initState();
    _firstLoad();
    _controller = ScrollController()..addListener(_loadMore);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          centerTitle: true,
          title: const Text(
            'Transactions History',
            style: TextStyle(color: Colors.white),
          ),
          leading: IconButton(
              onPressed: () {
                Navigator.of(context).maybePop();
                Navigator.of(context, rootNavigator: true).pop(context);
                },
              icon: Icon(
                Icons.arrow_back,
                color: Colors.white,
              )),
        ),
        body: _isFirstLoadRunning
            ? const Center(
                child: CircularProgressIndicator(),
              )
            : Column(
                children: [
                  Expanded(
                    child: ListView.builder(
                      physics: BouncingScrollPhysics(),
                      itemCount: _posts.length,
                      controller: _controller,
                      itemBuilder: (_, index) => Container(
                        margin: EdgeInsets.only(left: 10, right: 10),
                        height: 100,
                        color: Colors.white,
                        child: Center(
                          child: Container(
                            height: 80,
                            width: double.infinity,
                            child: Card(
                                child: Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Row(
                                  children: [
                                    Container(
                                      margin:
                                          EdgeInsets.only(left: 10, right: 10),
                                      height: 70,
                                      width: 100,
                                      // color: Colors.grey,
                                      child: Column(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.start,
                                        children: [
                                          Text("ID"),
                                          Text("Full Name"),
                                          Text("Amount"),
                                          Text("Date")
                                        ],
                                      ),
                                    ),
                                    Container(
                                      height: 70,
                                      width: 210,
                                      // color: Colors.green,
                                      child: Column(
                                        crossAxisAlignment:
                                            CrossAxisAlignment.start,
                                        children: [
                                          Text(_posts[index]['id'].toString()),
                                          Text(_posts[index]['full_name']
                                              .toString()),
                                          Text(_posts[index]['currency']
                                                  .toString() +
                                              " " +
                                              _posts[index]['amount']
                                                  .toString()),
                                          Text(_posts[index]['created_at']
                                              .toString()),
                                        ],
                                      ),
                                    ),
                                  ],
                                ),

                                // Text(snapshot.data![index].id.toString()),
                                // Text(snapshot.data![index].full_name.toString()),
                                // Text(snapshot.data![index].currency.toString()+" "+snapshot.data![index].amount.toString()),
                                // Text(snapshot.data![index].created_at.toString()),
                              ],
                            )),
                          ),
                        ),
                      ),
                    ),
                  ),
                  if (_isLoadMoreRunning == true)
                     Padding(
                      padding: EdgeInsets.only(top: 10, bottom: 15),
                      child: Center(
                        child: CircularProgressIndicator(),
                        // child: shimmerr(),
                      ),
                    ),
                  if (_hasNextPage == false)
                    Container(
                      padding: const EdgeInsets.only(top: 5, bottom: 5),
                      color: Colors.lightBlue,
                      child: const Center(
                        child: Text(
                          'You have fetched all of the content',
                          style: TextStyle(fontWeight: FontWeight.w500),
                        ),
                      ),
                    ),
                ],
              ));
  }

}





